package com.qq.qcloud.tencent_im_sdk_plugin.manager;

import com.qq.qcloud.tencent_im_sdk_plugin.util.CommonUtil;
import com.tencent.imsdk.v2.V2TIMCallback;
import com.tencent.imsdk.v2.V2TIMManager;
import com.tencent.imsdk.v2.V2TIMMessage;
import com.tencent.imsdk.v2.V2TIMOfflinePushInfo;
import com.tencent.imsdk.v2.V2TIMSignalingListener;

import java.util.HashMap;
import java.util.List;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;

public class SignalingManager {
    private static MethodChannel channel;
    public SignalingManager(MethodChannel _channel){
        SignalingManager.channel = _channel;
    }

    public void addSignalingListener(MethodCall methodCall, final MethodChannel.Result result){
        V2TIMManager.getSignalingManager().addSignalingListener(new V2TIMSignalingListener() {
            @Override
            public void onReceiveNewInvitation(String inviteID, String inviter, String groupID, List<String> inviteeList, String data) {
                HashMap<Object,Object> res = new HashMap<Object,Object>();
                res.put("inviteID",inviteID);
                res.put("inviter",inviter);
                res.put("groupID",groupID);
                res.put("inviteeList",inviteeList);
                res.put("data",data);
                makeSignalingListenerEventData("onReceiveNewInvitation",res);
            }

            @Override
            public void onInviteeAccepted(String inviteID, String invitee, String data) {
                HashMap<Object,Object> res = new HashMap<Object,Object>();
                res.put("inviteID",inviteID);
                res.put("invitee",invitee);
                res.put("data",data);
                makeSignalingListenerEventData("onReceiveNewInvitation",res);
            }

            @Override
            public void onInviteeRejected(String inviteID, String invitee, String data) {
                HashMap<Object,Object> res = new HashMap<Object,Object>();
                res.put("inviteID",inviteID);
                res.put("invitee",invitee);
                res.put("data",data);
                makeSignalingListenerEventData("onReceiveNewInvitation",res);
            }

            @Override
            public void onInvitationCancelled(String inviteID, String inviter, String data) {
                HashMap<Object,Object> res = new HashMap<Object,Object>();
                res.put("inviteID",inviteID);
                res.put("inviter",inviter);
                res.put("data",data);
                makeSignalingListenerEventData("onReceiveNewInvitation",res);
            }

            @Override
            public void onInvitationTimeout(String inviteID, List<String> inviteeList) {
                HashMap<Object,Object> res = new HashMap<Object,Object>();
                res.put("inviteID",inviteID);
                res.put("inviteeList",inviteeList);
                makeSignalingListenerEventData("onReceiveNewInvitation",res);
            }
        });
    }
    public void removeSignalingListener(MethodCall methodCall, final MethodChannel.Result result){
        V2TIMManager.getSignalingManager().removeSignalingListener(new V2TIMSignalingListener() {
            @Override
            public void onReceiveNewInvitation(String inviteID, String inviter, String groupID, List<String> inviteeList, String data) {
            }

            @Override
            public void onInviteeAccepted(String inviteID, String invitee, String data) {
            }

            @Override
            public void onInviteeRejected(String inviteID, String invitee, String data) {
            }

            @Override
            public void onInvitationCancelled(String inviteID, String inviter, String data) {
            }

            @Override
            public void onInvitationTimeout(String inviteID, List<String> inviteeList) {
            }
        });
    }

    private <T> void  makeSignalingListenerEventData(String type,T data){
        CommonUtil.emitEvent(SignalingManager.channel,"signalingListener",type,data);
    }
    public void invite(MethodCall methodCall, final MethodChannel.Result result){
        String invitee = CommonUtil.getParam(methodCall,result,"invitee");
        String data = CommonUtil.getParam(methodCall,result,"data");
        int timeout = CommonUtil.getParam(methodCall,result,"timeout");
        boolean onlineUserOnly = CommonUtil.getParam(methodCall,result,"onlineUserOnly");

        V2TIMManager.getSignalingManager().invite(invitee,data,onlineUserOnly,new V2TIMOfflinePushInfo(),timeout,new V2TIMCallback(){

            @Override
            public void onError(int i, String s) {
                CommonUtil.returnError(result,i,s);
            }

            @Override
            public void onSuccess() {
                CommonUtil.returnSuccess(result,null);
            }
        });

    }
    public void inviteInGroup(MethodCall methodCall, final MethodChannel.Result result){
        String groupID = CommonUtil.getParam(methodCall,result,"groupID");
        List<String> inviteeList = CommonUtil.getParam(methodCall,result,"inviteeList");
        String data = CommonUtil.getParam(methodCall,result,"data");
        int timeout = CommonUtil.getParam(methodCall,result,"timeout");
        boolean onlineUserOnly = CommonUtil.getParam(methodCall,result,"onlineUserOnly");
        V2TIMManager.getSignalingManager().inviteInGroup(groupID,inviteeList,data ,onlineUserOnly,timeout,new V2TIMCallback() {
            @Override
            public void onError(int i, String s) {
                CommonUtil.returnError(result,i,s);
            }

            @Override
            public void onSuccess() {
                CommonUtil.returnSuccess(result,null);
            }
        });
    }
    public void cancel(MethodCall methodCall, final MethodChannel.Result result){
        String inviteID = CommonUtil.getParam(methodCall,result,"inviteID");
        String data = CommonUtil.getParam(methodCall,result,"data");
        V2TIMManager.getSignalingManager().cancel(inviteID,data ,new V2TIMCallback() {
            @Override
            public void onError(int i, String s) {
                CommonUtil.returnError(result,i,s);
            }

            @Override
            public void onSuccess() {
                CommonUtil.returnSuccess(result,null);
            }
        });
    }
    public void accept(MethodCall methodCall, final MethodChannel.Result result){
        String inviteID = CommonUtil.getParam(methodCall,result,"inviteID");
        String data = CommonUtil.getParam(methodCall,result,"data");
        V2TIMManager.getSignalingManager().accept(inviteID,data ,new V2TIMCallback() {
            @Override
            public void onError(int i, String s) {
                CommonUtil.returnError(result,i,s);
            }

            @Override
            public void onSuccess() {
                CommonUtil.returnSuccess(result,null);
            }
        });
    }
    public void reject(MethodCall methodCall, final MethodChannel.Result result){
        String inviteID = CommonUtil.getParam(methodCall,result,"inviteID");
        String data = CommonUtil.getParam(methodCall,result,"data");
        V2TIMManager.getSignalingManager().reject(inviteID,data ,new V2TIMCallback() {
            @Override
            public void onError(int i, String s) {
                CommonUtil.returnError(result,i,s);
            }

            @Override
            public void onSuccess() {
                CommonUtil.returnSuccess(result,null);
            }
        });
    }
    public void getSignalingInfo(MethodCall methodCall, final MethodChannel.Result result){
        V2TIMManager.getSignalingManager().getSignalingInfo(new V2TIMMessage());
    }
    public void addInvitedSignaling(MethodCall methodCall, final MethodChannel.Result result){
        //这个方法可能被删掉了
    }
}
