import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:tencent_im_sdk_plugin/models/v2_tim_callback.dart';

/// 提供离线推送相关的接口
///
/// [setOfflinePushConfig]设置离线推送配置信息
///
/// [doBackground]APP 检测到应用退后台时可以调用此接口，可以用作桌面应用角标的初始化未读数量
///
/// [doForeground]APP 检测到应用进前台时可以调用此接口
///
/// {@category Manager}
///
class V2TIMAPNSManager {
  ///@nodoc
  MethodChannel _channel;

  ///@nodoc
  V2TIMAPNSManager(channel) {
    _channel = channel;
  }

  ///   设置离线推送配置信息
  ///
  /// 参数
  ///
  /// ```
  /// config	离线推送配置
  /// callback	回调
  /// ```
  ///
  Future<V2TimCallback> setAPNS({
    @required int businessID,
    String token,
  }) async {
    return V2TimCallback.fromJson(
      formatJson(
        await _channel.invokeMethod(
          "setAPNS",
          buildParam(
            {
              "businessID": businessID,
              "token": token
            },
          ),
        ),
      ),
    );
  }

  ///@nodoc
  Map buildParam(Map param) {
    param["TIMManagerName"] = "APNSManager";
    return param;
  }

  ///@nodoc
  formatJson(jsonSrc) {
    return json.decode(json.encode(jsonSrc));
  }
}
